/************************************************************************/
/*  External variable for sac2ms.					*/
/*									*/
/*	Cedric de La Beaujardiere					*/
/*	Seismological Laboratory					*/
/*	University of California, Berkeley				*/
/*	doug@seismo.berkeley.edu					*/
/*									*/
/************************************************************************/

/*
 * Copyright (c) 1996-2004 The Regents of the University of California.
 * All Rights Reserved.
 * 
 * Permission to use, copy, modify, and distribute this software and its
 * documentation for educational, research and non-profit purposes,
 * without fee, and without a written agreement is hereby granted,
 * provided that the above copyright notice, this paragraph and the
 * following three paragraphs appear in all copies.
 * 
 * Permission to incorporate this software into commercial products may
 * be obtained from the Office of Technology Licensing, 2150 Shattuck
 * Avenue, Suite 510, Berkeley, CA  94704.
 * 
 * IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY PARTY
 * FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES,
 * INCLUDING LOST PROFITS, ARISING OUT OF THE USE OF THIS SOFTWARE AND
 * ITS DOCUMENTATION, EVEN IF THE UNIVERSITY OF CALIFORNIA HAS BEEN
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 * THE UNIVERSITY OF CALIFORNIA SPECIFICALLY DISCLAIMS ANY WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.  THE SOFTWARE
 * PROVIDED HEREUNDER IS ON AN "AS IS" BASIS, AND THE UNIVERSITY OF
 * CALIFORNIA HAS NO OBLIGATIONS TO PROVIDE MAINTENANCE, SUPPORT,
 * UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
 */

#ifndef externals_h
#define externals_h

/*	$Id: externals.h,v 1.8 2006/11/14 23:16:53 doug Exp $ 	*/

#ifndef seg2_struct_h
#include "seg2_struct.h"
#endif

/* float data are multiplied by this value to get into integer form. */
#define DFLT_FLOAT_SCALING_FACTOR 50000 

#define		FATAL(str)	{ fprintf(stderr,"%s\n",str); exit(1); }
#define		MAX_BLKSIZE	8192
	
#ifndef	EXTERN
#define	EXTERN	extern
#endif	/* EXTERN */

typedef struct _mapentry {
   /* char seg2_station[6];*/
    int seg2_channel;
    char seed_station[6];
    char seed_network[3];
    char seed_channel[4];
    char seed_location[3];
}   MAPENTRY;


/************************************************************************/
/*  External variables.							*/
/************************************************************************/
EXTERN char *cmdname;
EXTERN char *infile;
EXTERN char *outfile;
EXTERN char *mapfile;
EXTERN FILE *input;
EXTERN FILE *output;
EXTERN FILE *info;
EXTERN char station[6];	  /* explicit station name.		*/
EXTERN char channel[4];	  /* explicit channel name.		*/
EXTERN char network[3];	  /* explicit network name.		*/
EXTERN char location[3];  /* explicit location name.		*/
EXTERN char acq_date[11]; /* explicit ACQUISITION_DATE value.	*/
EXTERN int bin_flag;
EXTERN int summary_flag;
EXTERN KEY_VAL_PAIR **orfv;
EXTERN KEY_VAL_PAIR **ortv;
EXTERN int orfc;
EXTERN int ortc;
EXTERN int clock_quality;
EXTERN int no_output;

EXTERN MAPENTRY *map;
extern char *syntax[];


#ifdef	DEFINE
int clock_quality = 100;			/* Dflt clock quality	*/
int blksize = DEFAULT_BLKSIZE;			/* MiniSEED blksize.	*/
int output_format = DEFAULT_OUTPUT_FORMAT;	/* MiniSEED data format.*/
double float_scaling_factor = DFLT_FLOAT_SCALING_FACTOR;

int dbgflag = 0;

#else
EXTERN int	clock_quality;
EXTERN int	blksize;	/* MiniSEED blksize.			*/
EXTERN int	output_format;	/* MiniSEED data format.		*/
EXTERN int      dbgflag;        /* debug flag using DBG_FLAGS bit set   */
EXTERN double	float_scaling_factor;
#endif

typedef enum debug_flags {
  DBG_FLAG_FD_TRACES      =  1, /* file descriptor #traces and trace ptrs. */
  DBG_FLAG_FD_VALS        =  2, /* file descriptor values */
  DBG_FLAG_FD_STRS_PARSED =  4, /* file descriptor strings (keywd=value) */
  DBG_FLAG_TD_VALS        =  8, /* trace descriptor values */
  DBG_FLAG_TD_STRS_PARSED = 16, /* trace descriptors strings (keywd=value) */
  DBG_FLAG_TD_MIN_MAX	  = 32, /* trace data min,max data values. */
  DBG_FLAG_TD_DUMP	  = 64,	/* trace data raw data  values. */
  ALL_DBG_FLAGS_ON = 1 | 2 | 4 | 8 | 16 | 32 | 64,
} DBG_FLAGS;

SEG2_KEYWORDS *file_desc_keywords;
SEG2_KEYWORDS *trace_desc_keywords;
SEG2_KEYWORDS *override_file_desc_keywords;
SEG2_KEYWORDS *override_trace_desc_keywords;

#endif /* end of #ifndef externals_h */
