#include <stdio.h>
#include <strings.h>

#include "seg2_struct.h"
#include "externals.h"
#include "procs.h"

/************************************************************************/
/*  override_file_key_val:						*/
/*	Function to update file key_vals with command line overrides.	*/
/*  return: 0 on success, -1 on failure.				*/
/************************************************************************/
int override_file_key_val (SEG2FILEDESC *file_desc, KEY_VAL_PAIR **orfv, int orfc)
{
    KEY_VAL_PAIR *kp;
    int i;
    if (orfv == NULL || orfc == 0) return (0);
    for (i=0; i<orfc; i++) {
	int n = 0;
	kp=orfv[i];
	n = override_known_key_val (file_desc->known_keys, NUM_OF_FD_KEYWORDS, kp, file_desc_keywords);
	if (n == 0) {
	    n = override_unknown_key_val (&(file_desc->unknown_keys), &(file_desc->unknown_keys_count), kp);
	    if (n == 0) {
		fprintf (stderr, "Error: overriding file_key\n");
		exit(1);
	    }
	}
    }
    return (0);
}

/************************************************************************/
/*  override_trace_key_val:						*/
/*	Function to update trace key_vals with command line overrides.	*/
/*  return: 0 on success, -1 on failure.				*/
/************************************************************************/
int override_trace_key_val (SEG2TRACEDESC *trace_desc, KEY_VAL_PAIR **orfv, int orfc)
{
    KEY_VAL_PAIR *kp;
    int i;
    if (ortv == NULL || orfc == 0) return (0);
    for (i=0; i<orfc; i++) {
	int n = 0;
	kp=orfv[i];
	n = override_known_key_val (trace_desc->known_keys, NUM_OF_TD_KEYWORDS, kp, trace_desc_keywords);
	if (n == 0) {
	    n = override_unknown_key_val (&(trace_desc->unknown_keys), &(trace_desc->unknown_keys_count), kp);
	    if (n == 0) {
		fprintf (stderr, "Error: overriding trace_key\n");
		exit(1);
	    }
	}
    }
    return (0);
}

/************************************************************************/
/*  override_known_key_val:						*/
/*	Given a ptr to KEY_VAL_PAIR array and length of array,		*/
/*	search for the specified key_val_pair, and if found, replace it	*/
/*	with a copy of the new KEY_VAL_PAIR.				*/
/*  return: 0 if key not updated, 1 if key was updated.			*/
/************************************************************************/
int override_known_key_val (KEY_VAL_PAIR *ka[], int n, KEY_VAL_PAIR *kp, SEG2_KEYWORDS *kw)
{
    int i;
    for (i=0; i<n; i++) {
	if (ka[i] != NULL && strcmp (ka[i]->key, kp->key) == 0) {
	    free(ka[i]->val);
	    ka[i]->val = (char *)malloc (kp->val_length*sizeof(char));
	    if (ka[i]->val == NULL) {
		fprintf (stderr, "Error: mallocing key for key override\n");
		exit(1);
	    }
	    strncpy (ka[i]->val,kp->val,kp->val_length);
	    ka[i]->val_length = kp->val_length;
	    ka[i]->val[kp->val_length] = '\0';
	    return (1);
	}
    }
    /* Did not find key. */
    /* If this is really a known key, find it in the known key table,	*/
    /* use it supposed index, and add it to the table.			*/
    for (i=0; i<kw->max_index; i++) {
	if (kw->keywords[i] != NULL && strcmp (kw->keywords[i], kp->key) == 0) {
	    ka[i] = (KEY_VAL_PAIR *)malloc(sizeof(KEY_VAL_PAIR));
	    if (ka[i] == NULL) {
		fprintf (stderr, "Error: mallocing key_val_pair for key override\n");
		exit(1);
	    }
	    ka[i]->key = (char*)malloc (kp->key_length*sizeof(char));
	    if (ka[i]->key == NULL) {
		fprintf (stderr, "Error: mallocing key for key override\n");
		exit(1);
	    }
	    ka[i]->val = (char*)malloc (kp->val_length*sizeof(char));
	    if (ka[i]->val == NULL) {
		fprintf (stderr, "Error: mallocing val for key override\n");
		exit(1);
	    }
	    strncpy (ka[i]->key,kp->key,kp->key_length);
	    ka[i]->key_length = kp->key_length;
	    ka[i]->key[kp->key_length] = '\0';
	    strncpy (ka[i]->val,kp->val,kp->val_length);
	    ka[i]->val_length = kp->val_length;
	    ka[i]->val[kp->val_length] = '\0';
	    return (1);
	}
    }
    return (0);
}

/************************************************************************/
/*  override_unknown_key_val:						*/
/*	Given a ptr to KEY_VAL_PAIR list and ptr to # of list elements,	*/
/*	search for the specified key_val_pair, and if found, replace it	*/
/*	with a copy of the new KEY_VAL_PAIR.				*/
/*	If flag != 0, add the new KEY_VAL_PAIR to the list if it was	*/
/*	not found.							*/
/*  return: 0 if key not added / updated, 1 if key was added / updated.	*/
/************************************************************************/
int override_unknown_key_val (KEY_VAL_PAIR ***ap, int *n, KEY_VAL_PAIR *kp)
{
    int i;
    for (i=0; i<*n; i++) {
	if (strcmp ((*ap)[i]->key, kp->key) == 0) {
	    free((*ap)[i]->val);
	    (*ap)[i]->val = (char *)malloc (kp->val_length*sizeof(char));
	    if ((*ap)[i]->val == NULL) {
		fprintf (stderr, "Error: mallocing key for key override\n");
		exit(1);
	    }
	    strncpy ((*ap)[i]->val,kp->val,kp->val_length);
	    (*ap)[i]->val_length = kp->val_length;
	    (*ap)[i]->val[kp->val_length] = '\0';
	    return (1);
	}
    }
    /* Did not find key.  Add key to this list. */
    i = *n;
    ap = (KEY_VAL_PAIR ***)realloc(ap,((*n)+1) * sizeof(KEY_VAL_PAIR **));
    if (ap == NULL) {
	fprintf (stderr, "Error: realloc ptr to key_val_pair list\n");
	exit(1);
    }
    (*ap)[i] = (KEY_VAL_PAIR *)malloc (sizeof(KEY_VAL_PAIR));
    if (*ap[i] == NULL) {
	fprintf (stderr, "Error: malloc key_val_pair\n");
	exit(1);
    }
    (*ap)[i]->key = (char *)malloc(kp->key_length+1 * sizeof(char));
    if ((*ap)[i]->key == NULL) {
	fprintf (stderr, "Error: malloc key_storage\n");
	exit(1);
    }
    (*ap)[i]->val = (char *)malloc(kp->val_length+1 * sizeof(char));
    if ((*ap)[i]->val == NULL) {
	fprintf (stderr, "Error: malloc val_storage\n");
	exit(1);
    }
    strncpy((*ap)[i]->key, kp->key, kp->key_length);
    (*ap)[i]->key[kp->key_length] = '\0';
    (*ap)[i]->key_length = kp->key_length;
    strncpy((*ap)[i]->val, kp->val, kp->val_length);
    (*ap)[i]->val[kp->val_length] = '\0';
    (*ap)[i]->val_length = kp->val_length;
    return (1);
}

/************************************************************************/
/*  key_val_store:							*/
/*	Parse a key_val in the specified string, and add it to the	*/
/*	KEY_VAL_PAIR list.						*/
/*  return: 0 on sucess, -1 on failure.					*/
/************************************************************************/
int key_val_store (char *str, KEY_VAL_PAIR ***p, int *n)
{
    KEY_VAL_PAIR **av = *p;
    int ac = *n;
    KEY_VAL_PAIR *k;
    k = key_val_define(str);
    if (k == NULL) exit(1);
    av = (KEY_VAL_PAIR **)realloc(av, ((ac+1) * sizeof(KEY_VAL_PAIR *)));
    if (p == NULL) {
	fprintf (stderr, "Error: reallocing key_val_pair list\n");
	exit(1);
    }
    av[ac] = k;
    ++ac;
    *p = av;
    *n = ac;
    return (0);
}


/************************************************************************/
/*  key_val_define:							*/
/*	Parse an override string in the format of key=value		*/
/*	and return a pointer to a KEY_VAL_PAIR.				*/
/*	Return NULL on error.						*/
/************************************************************************/
KEY_VAL_PAIR *key_val_define (char *str)
{
    KEY_VAL_PAIR *key_val = NULL;
    char *key;
    char *val;
    char *p;
    int key_len;
    int val_len;

    key = str;
    val = strchr (str,'=');
    if (val == NULL) return (key_val);
    key_len = val-str;
    val_len = strlen (++val);

    /* Allocate memory for the KEY_VAL_PAIR structure */
    key_val = (KEY_VAL_PAIR *)malloc(sizeof(KEY_VAL_PAIR));
    if (key_val == NULL) {
	fprintf(stderr,
		"key_val_define: Can't allocate memory block for key_val structure. current keyword: %s\n",
		key);
	return(NULL);
    } 
    /* Allocate memory for the KEY_VAL_PAIR structure's keyword array */
    key_val->key = (char *)malloc(sizeof(char) * (key_len + 1));
    if (key_val->key == NULL) {
	fprintf(stderr,
		"key_val_define: Can't allocate memory block for key_val's key array. current keyword: %s\n",
		key);
	free(key_val);
	return(NULL);
    }
    /* Allocate memory for the KEY_VAL_PAIR structure's value array */
    key_val->val = (char *)malloc(sizeof(char) * (val_len + 1));
    if (key_val->val == NULL) {
	fprintf(stderr,
		"key_val_define: Can't allocate memory block for key_val's val array. current keyword: %s\n",
		key);
	free(key_val->key);
	free(key_val);
	return(NULL);
    }

    /* Copy the keyword and value into the KEY_VAL_PAIR structure */
    strncpy(key_val->key, key, key_len);
    strncpy(key_val->val, val, val_len);
    key_val->key[key_len] = '\0';
    key_val->val[val_len] = '\0';
    key_val->key_length = key_len;
    key_val->val_length = val_len;

    return(key_val); 
}
